# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.firewall.waf import (
    override_list_params,
    override_create_params,
    override_update_params,
)
from ....types.firewall.waf.override import Override
from ....types.firewall.waf.override_url import OverrideURL
from ....types.firewall.waf.waf_rule_param import WAFRuleParam
from ....types.firewall.waf.rewrite_action_param import RewriteActionParam
from ....types.firewall.waf.override_delete_response import OverrideDeleteResponse

__all__ = ["OverridesResource", "AsyncOverridesResource"]


class OverridesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OverridesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OverridesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OverridesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OverridesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def create(
        self,
        *,
        zone_id: str,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Creates a URI-based WAF override for a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/firewall/waf/overrides", zone_id=zone_id),
            body=maybe_transform({"urls": urls}, override_create_params.OverrideCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )

    @typing_extensions.deprecated("deprecated")
    def update(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        id: str,
        rewrite_action: RewriteActionParam,
        rules: WAFRuleParam,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Updates an existing URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          id: Defines an identifier.

          rewrite_action: Specifies that, when a WAF rule matches, its configured action will be replaced
              by the action configured in this object.

          rules: An object that allows you to override the action of specific WAF rules. Each key
              of this object must be the ID of a WAF rule, and each value must be a valid WAF
              action. Unless you are disabling a rule, ensure that you also enable the rule
              group that this WAF rule belongs to. When creating a new URI-based WAF override,
              you must provide a `groups` object or a `rules` object.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return self._put(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            body=maybe_transform(
                {
                    "id": id,
                    "rewrite_action": rewrite_action,
                    "rules": rules,
                    "urls": urls,
                },
                override_update_params.OverrideUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        zone_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Override]:
        """
        Fetches the URI-based WAF overrides in a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          page: The page number of paginated results.

          per_page: The number of WAF overrides per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/waf/overrides", zone_id=zone_id),
            page=SyncV4PagePaginationArray[Override],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    override_list_params.OverrideListParams,
                ),
            ),
            model=Override,
        )

    @typing_extensions.deprecated("deprecated")
    def delete(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OverrideDeleteResponse]:
        """
        Deletes an existing URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OverrideDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OverrideDeleteResponse]], ResultWrapper[OverrideDeleteResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Fetches the details of a URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )


class AsyncOverridesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOverridesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOverridesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOverridesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOverridesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def create(
        self,
        *,
        zone_id: str,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Creates a URI-based WAF override for a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/firewall/waf/overrides", zone_id=zone_id),
            body=await async_maybe_transform({"urls": urls}, override_create_params.OverrideCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )

    @typing_extensions.deprecated("deprecated")
    async def update(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        id: str,
        rewrite_action: RewriteActionParam,
        rules: WAFRuleParam,
        urls: SequenceNotStr[OverrideURL],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Updates an existing URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          id: Defines an identifier.

          rewrite_action: Specifies that, when a WAF rule matches, its configured action will be replaced
              by the action configured in this object.

          rules: An object that allows you to override the action of specific WAF rules. Each key
              of this object must be the ID of a WAF rule, and each value must be a valid WAF
              action. Unless you are disabling a rule, ensure that you also enable the rule
              group that this WAF rule belongs to. When creating a new URI-based WAF override,
              you must provide a `groups` object or a `rules` object.

          urls: The URLs to include in the current WAF override. You can use wildcards. Each
              entered URL will be escaped before use, which means you can only use simple
              wildcard patterns.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return await self._put(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            body=await async_maybe_transform(
                {
                    "id": id,
                    "rewrite_action": rewrite_action,
                    "rules": rules,
                    "urls": urls,
                },
                override_update_params.OverrideUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        *,
        zone_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Override, AsyncV4PagePaginationArray[Override]]:
        """
        Fetches the URI-based WAF overrides in a zone.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          page: The page number of paginated results.

          per_page: The number of WAF overrides per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/firewall/waf/overrides", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[Override],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    override_list_params.OverrideListParams,
                ),
            ),
            model=Override,
        )

    @typing_extensions.deprecated("deprecated")
    async def delete(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OverrideDeleteResponse]:
        """
        Deletes an existing URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OverrideDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OverrideDeleteResponse]], ResultWrapper[OverrideDeleteResponse]),
        )

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        overrides_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Override:
        """
        Fetches the details of a URI-based WAF override.

        **Note:** Applies only to the
        [previous version of WAF managed rules](https://developers.cloudflare.com/support/firewall/managed-rules-web-application-firewall-waf/understanding-waf-managed-rules-web-application-firewall/).

        Args:
          zone_id: Defines an identifier.

          overrides_id: The unique identifier of the WAF override.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not overrides_id:
            raise ValueError(f"Expected a non-empty value for `overrides_id` but received {overrides_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/firewall/waf/overrides/{overrides_id}", zone_id=zone_id, overrides_id=overrides_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Override]._unwrapper,
            ),
            cast_to=cast(Type[Override], ResultWrapper[Override]),
        )


class OverridesResourceWithRawResponse:
    def __init__(self, overrides: OverridesResource) -> None:
        self._overrides = overrides

        self.create = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                overrides.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                overrides.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                overrides.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                overrides.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                overrides.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncOverridesResourceWithRawResponse:
    def __init__(self, overrides: AsyncOverridesResource) -> None:
        self._overrides = overrides

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                overrides.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                overrides.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                overrides.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                overrides.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                overrides.get,  # pyright: ignore[reportDeprecated],
            )
        )


class OverridesResourceWithStreamingResponse:
    def __init__(self, overrides: OverridesResource) -> None:
        self._overrides = overrides

        self.create = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                overrides.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                overrides.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                overrides.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                overrides.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                overrides.get,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncOverridesResourceWithStreamingResponse:
    def __init__(self, overrides: AsyncOverridesResource) -> None:
        self._overrides = overrides

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                overrides.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                overrides.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                overrides.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                overrides.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                overrides.get,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["RegistrationEditParams"]


class RegistrationEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    auto_renew: bool
    """
    Enable or disable automatic renewal. Setting this field to `true` authorizes
    Cloudflare to charge the account's default payment method up to 30 days before
    domain expiry to renew the domain automatically. Renewal pricing may change over
    time based on registry pricing.
    """

    prefer: Annotated[Literal["respond-async"], PropertyInfo(alias="Prefer")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MTLSCertificateListParams"]


class MTLSCertificateListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    type: List[Literal["custom", "gateway_managed", "access_managed"]]
    """Filters results by certificate type. Multiple types can be comma-separated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .applications.access_rule import AccessRule

__all__ = ["GroupGetResponse"]


class GroupGetResponse(BaseModel):
    id: Optional[str] = None
    """UUID."""

    exclude: Optional[List[AccessRule]] = None
    """Rules evaluated with a NOT logical operator.

    To match a policy, a user cannot meet any of the Exclude rules.
    """

    include: Optional[List[AccessRule]] = None
    """Rules evaluated with an OR logical operator.

    A user needs to meet only one of the Include rules.
    """

    is_default: Optional[List[AccessRule]] = None
    """Rules evaluated with an AND logical operator.

    To match a policy, a user must meet all of the Require rules.
    """

    name: Optional[str] = None
    """The name of the Access group."""

    require: Optional[List[AccessRule]] = None
    """Rules evaluated with an AND logical operator.

    To match a policy, a user must meet all of the Require rules.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .host import Host as Host
from .pool import Pool as Pool
from .rules import Rules as Rules
from .header import Header as Header
from .origin import Origin as Origin
from .monitor import Monitor as Monitor
from .rules_param import RulesParam as RulesParam
from .check_region import CheckRegion as CheckRegion
from .header_param import HeaderParam as HeaderParam
from .origin_param import OriginParam as OriginParam
from .default_pools import DefaultPools as DefaultPools
from .load_balancer import LoadBalancer as LoadBalancer
from .load_shedding import LoadShedding as LoadShedding
from .monitor_group import MonitorGroup as MonitorGroup
from .filter_options import FilterOptions as FilterOptions
from .origin_steering import OriginSteering as OriginSteering
from .random_steering import RandomSteering as RandomSteering
from .steering_policy import SteeringPolicy as SteeringPolicy
from .adaptive_routing import AdaptiveRouting as AdaptiveRouting
from .pool_edit_params import PoolEditParams as PoolEditParams
from .pool_list_params import PoolListParams as PoolListParams
from .session_affinity import SessionAffinity as SessionAffinity
from .location_strategy import LocationStrategy as LocationStrategy
from .pool_create_params import PoolCreateParams as PoolCreateParams
from .pool_update_params import PoolUpdateParams as PoolUpdateParams
from .region_list_params import RegionListParams as RegionListParams
from .search_list_params import SearchListParams as SearchListParams
from .load_shedding_param import LoadSheddingParam as LoadSheddingParam
from .monitor_edit_params import MonitorEditParams as MonitorEditParams
from .notification_filter import NotificationFilter as NotificationFilter
from .region_get_response import RegionGetResponse as RegionGetResponse
from .filter_options_param import FilterOptionsParam as FilterOptionsParam
from .pool_delete_response import PoolDeleteResponse as PoolDeleteResponse
from .preview_get_response import PreviewGetResponse as PreviewGetResponse
from .region_list_response import RegionListResponse as RegionListResponse
from .search_list_response import SearchListResponse as SearchListResponse
from .monitor_create_params import MonitorCreateParams as MonitorCreateParams
from .monitor_update_params import MonitorUpdateParams as MonitorUpdateParams
from .origin_steering_param import OriginSteeringParam as OriginSteeringParam
from .pool_bulk_edit_params import PoolBulkEditParams as PoolBulkEditParams
from .random_steering_param import RandomSteeringParam as RandomSteeringParam
from .adaptive_routing_param import AdaptiveRoutingParam as AdaptiveRoutingParam
from .location_strategy_param import LocationStrategyParam as LocationStrategyParam
from .monitor_delete_response import MonitorDeleteResponse as MonitorDeleteResponse
from .load_balancer_edit_params import LoadBalancerEditParams as LoadBalancerEditParams
from .monitor_group_edit_params import MonitorGroupEditParams as MonitorGroupEditParams
from .notification_filter_param import NotificationFilterParam as NotificationFilterParam
from .load_balancer_create_params import LoadBalancerCreateParams as LoadBalancerCreateParams
from .load_balancer_update_params import LoadBalancerUpdateParams as LoadBalancerUpdateParams
from .monitor_group_create_params import MonitorGroupCreateParams as MonitorGroupCreateParams
from .monitor_group_update_params import MonitorGroupUpdateParams as MonitorGroupUpdateParams
from .session_affinity_attributes import SessionAffinityAttributes as SessionAffinityAttributes
from .load_balancer_delete_response import LoadBalancerDeleteResponse as LoadBalancerDeleteResponse
from .session_affinity_attributes_param import SessionAffinityAttributesParam as SessionAffinityAttributesParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Websocket"]


class Websocket(BaseModel):
    """
    WebSockets are open connections sustained between the client and the origin server. Inside a WebSockets connection, the client and the origin can pass data back and forth without having to reestablish sessions. This makes exchanging data within a WebSockets connection fast. WebSockets are often used for real-time applications such as live chat and gaming. For more information refer to [Can I use Cloudflare with Websockets](https://support.cloudflare.com/hc/en-us/articles/200169466-Can-I-use-Cloudflare-with-WebSockets-).
    """

    id: Literal["websockets"]
    """ID of the zone setting."""

    value: Literal["off", "on"]
    """Current value of the zone setting."""

    editable: Optional[Literal[True, False]] = None
    """
    Whether or not this setting can be modified for this zone (based on your
    Cloudflare plan level).
    """

    modified_on: Optional[datetime] = None
    """last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.security_center.insights.context_get_response import ContextGetResponse

__all__ = ["ContextResource", "AsyncContextResource"]


class ContextResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ContextResourceWithStreamingResponse(self)

    def get(
        self,
        issue_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ContextGetResponse]:
        """Returns the full context payload for an insight.

        This endpoint is used for
        insights with large payloads that are not included inline in the list response.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/security-center/insights/{issue_id}/context",
                account_id=account_id,
                issue_id=issue_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ContextGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ContextGetResponse]], ResultWrapper[ContextGetResponse]),
        )


class AsyncContextResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncContextResourceWithStreamingResponse(self)

    async def get(
        self,
        issue_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ContextGetResponse]:
        """Returns the full context payload for an insight.

        This endpoint is used for
        insights with large payloads that are not included inline in the list response.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/security-center/insights/{issue_id}/context",
                account_id=account_id,
                issue_id=issue_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ContextGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ContextGetResponse]], ResultWrapper[ContextGetResponse]),
        )


class ContextResourceWithRawResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.get = to_raw_response_wrapper(
            context.get,
        )


class AsyncContextResourceWithRawResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.get = async_to_raw_response_wrapper(
            context.get,
        )


class ContextResourceWithStreamingResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.get = to_streamed_response_wrapper(
            context.get,
        )


class AsyncContextResourceWithStreamingResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.get = async_to_streamed_response_wrapper(
            context.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.bgp.top import ase_get_params, ase_prefixes_params
from .....types.radar.bgp.top.ase_get_response import AseGetResponse
from .....types.radar.bgp.top.ase_prefixes_response import AsePrefixesResponse

__all__ = ["AsesResource", "AsyncAsesResource"]


class AsesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        prefix: SequenceNotStr[str] | Omit = omit,
        update_type: List[Literal["ANNOUNCEMENT", "WITHDRAWAL"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AseGetResponse:
        """
        Retrieves the top autonomous systems by BGP updates (announcements only).

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          name: Array of names used to label the series in the response.

          prefix: Filters results by BGP network prefix.

          update_type: Filters results by BGP update type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/top/ases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "name": name,
                        "prefix": prefix,
                        "update_type": update_type,
                    },
                    ase_get_params.AseGetParams,
                ),
                post_parser=ResultWrapper[AseGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AseGetResponse], ResultWrapper[AseGetResponse]),
        )

    def prefixes(
        self,
        *,
        country: str | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsePrefixesResponse:
        """
        Retrieves the full list of autonomous systems on the global routing table
        ordered by announced prefixes count. The data comes from public BGP MRT data
        archives and updates every 2 hours.

        Args:
          country: Alpha-2 country code.

          format: Format in which results will be returned.

          limit: Maximum number of ASes to return.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/top/ases/prefixes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country": country,
                        "format": format,
                        "limit": limit,
                    },
                    ase_prefixes_params.AsePrefixesParams,
                ),
                post_parser=ResultWrapper[AsePrefixesResponse]._unwrapper,
            ),
            cast_to=cast(Type[AsePrefixesResponse], ResultWrapper[AsePrefixesResponse]),
        )


class AsyncAsesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAsesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        prefix: SequenceNotStr[str] | Omit = omit,
        update_type: List[Literal["ANNOUNCEMENT", "WITHDRAWAL"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AseGetResponse:
        """
        Retrieves the top autonomous systems by BGP updates (announcements only).

        Args:
          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit: Limits the number of objects returned in the response.

          name: Array of names used to label the series in the response.

          prefix: Filters results by BGP network prefix.

          update_type: Filters results by BGP update type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/top/ases",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit": limit,
                        "name": name,
                        "prefix": prefix,
                        "update_type": update_type,
                    },
                    ase_get_params.AseGetParams,
                ),
                post_parser=ResultWrapper[AseGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AseGetResponse], ResultWrapper[AseGetResponse]),
        )

    async def prefixes(
        self,
        *,
        country: str | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsePrefixesResponse:
        """
        Retrieves the full list of autonomous systems on the global routing table
        ordered by announced prefixes count. The data comes from public BGP MRT data
        archives and updates every 2 hours.

        Args:
          country: Alpha-2 country code.

          format: Format in which results will be returned.

          limit: Maximum number of ASes to return.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/top/ases/prefixes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country": country,
                        "format": format,
                        "limit": limit,
                    },
                    ase_prefixes_params.AsePrefixesParams,
                ),
                post_parser=ResultWrapper[AsePrefixesResponse]._unwrapper,
            ),
            cast_to=cast(Type[AsePrefixesResponse], ResultWrapper[AsePrefixesResponse]),
        )


class AsesResourceWithRawResponse:
    def __init__(self, ases: AsesResource) -> None:
        self._ases = ases

        self.get = to_raw_response_wrapper(
            ases.get,
        )
        self.prefixes = to_raw_response_wrapper(
            ases.prefixes,
        )


class AsyncAsesResourceWithRawResponse:
    def __init__(self, ases: AsyncAsesResource) -> None:
        self._ases = ases

        self.get = async_to_raw_response_wrapper(
            ases.get,
        )
        self.prefixes = async_to_raw_response_wrapper(
            ases.prefixes,
        )


class AsesResourceWithStreamingResponse:
    def __init__(self, ases: AsesResource) -> None:
        self._ases = ases

        self.get = to_streamed_response_wrapper(
            ases.get,
        )
        self.prefixes = to_streamed_response_wrapper(
            ases.prefixes,
        )


class AsyncAsesResourceWithStreamingResponse:
    def __init__(self, ases: AsyncAsesResource) -> None:
        self._ases = ases

        self.get = async_to_streamed_response_wrapper(
            ases.get,
        )
        self.prefixes = async_to_streamed_response_wrapper(
            ases.prefixes,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Status"]

Status: TypeAlias = Literal["member", "invited"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "NamespaceSearchParams",
    "AISearchOptions",
    "AISearchOptionsCache",
    "AISearchOptionsQueryRewrite",
    "AISearchOptionsReranking",
    "AISearchOptionsRetrieval",
    "AISearchOptionsRetrievalBoostBy",
    "Message",
]


class NamespaceSearchParams(TypedDict, total=False):
    account_id: Required[str]

    aisearch_options: Required[Annotated[AISearchOptions, PropertyInfo(alias="ai_search_options")]]

    messages: Iterable[Message]

    query: str
    """A simple text query string.

    Alternative to 'messages' — provide either this or 'messages', not both.
    """


class AISearchOptionsCache(TypedDict, total=False):
    cache_threshold: Literal["super_strict_match", "close_enough", "flexible_friend", "anything_goes"]

    enabled: bool


class AISearchOptionsQueryRewrite(TypedDict, total=False):
    enabled: bool

    model: Literal[
        "@cf/meta/llama-3.3-70b-instruct-fp8-fast",
        "@cf/zai-org/glm-4.7-flash",
        "@cf/meta/llama-3.1-8b-instruct-fast",
        "@cf/meta/llama-3.1-8b-instruct-fp8",
        "@cf/meta/llama-4-scout-17b-16e-instruct",
        "@cf/qwen/qwen3-30b-a3b-fp8",
        "@cf/deepseek-ai/deepseek-r1-distill-qwen-32b",
        "@cf/moonshotai/kimi-k2-instruct",
        "@cf/google/gemma-3-12b-it",
        "@cf/google/gemma-4-26b-a4b-it",
        "@cf/moonshotai/kimi-k2.5",
        "anthropic/claude-3-7-sonnet",
        "anthropic/claude-sonnet-4",
        "anthropic/claude-opus-4",
        "anthropic/claude-3-5-haiku",
        "cerebras/qwen-3-235b-a22b-instruct",
        "cerebras/qwen-3-235b-a22b-thinking",
        "cerebras/llama-3.3-70b",
        "cerebras/llama-4-maverick-17b-128e-instruct",
        "cerebras/llama-4-scout-17b-16e-instruct",
        "cerebras/gpt-oss-120b",
        "google-ai-studio/gemini-2.5-flash",
        "google-ai-studio/gemini-2.5-pro",
        "grok/grok-4",
        "groq/llama-3.3-70b-versatile",
        "groq/llama-3.1-8b-instant",
        "openai/gpt-5",
        "openai/gpt-5-mini",
        "openai/gpt-5-nano",
        "",
    ]

    rewrite_prompt: str


class AISearchOptionsReranking(TypedDict, total=False):
    enabled: bool

    match_threshold: float

    model: Literal["@cf/baai/bge-reranker-base", ""]


class AISearchOptionsRetrievalBoostBy(TypedDict, total=False):
    field: Required[str]
    """Metadata field name to boost by.

    Use 'timestamp' for document freshness, or any custom_metadata field. Numeric
    and datetime fields support asc/desc directions; text/boolean fields support
    exists/not_exists.
    """

    direction: Literal["asc", "desc", "exists", "not_exists"]
    """Boost direction.

    'desc' = higher values rank higher (e.g. newer timestamps). 'asc' = lower values
    rank higher. 'exists' = boost chunks that have the field. 'not_exists' = boost
    chunks that lack the field. Optional ��� defaults to 'asc' for numeric/datetime
    fields, 'exists' for text/boolean fields.
    """


class AISearchOptionsRetrieval(TypedDict, total=False):
    boost_by: Iterable[AISearchOptionsRetrievalBoostBy]
    """Metadata fields to boost search results by.

    Overrides the instance-level boost_by config. Direction defaults to 'asc' for
    numeric/datetime fields, 'exists' for text/boolean fields. Fields must match
    'timestamp' or a defined custom_metadata field.
    """

    context_expansion: int

    filters: Dict[str, object]

    fusion_method: Literal["max", "rrf"]

    keyword_match_mode: Literal["and", "or"]
    """Controls which documents are candidates for BM25 scoring.

    'and' restricts candidates to documents containing all query terms; 'or'
    includes any document containing at least one term, ranked by BM25 relevance.
    Defaults to 'and'.
    """

    match_threshold: float

    max_num_results: int

    retrieval_type: Literal["vector", "keyword", "hybrid"]

    return_on_failure: bool


class AISearchOptions(TypedDict, total=False):
    instance_ids: Required[SequenceNotStr[str]]

    cache: AISearchOptionsCache

    query_rewrite: AISearchOptionsQueryRewrite

    reranking: AISearchOptionsReranking

    retrieval: AISearchOptionsRetrieval


class Message(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    content: Required[Optional[str]]

    role: Required[Literal["system", "developer", "user", "assistant", "tool"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dcv_delegation import (
    DCVDelegationResource,
    AsyncDCVDelegationResource,
    DCVDelegationResourceWithRawResponse,
    AsyncDCVDelegationResourceWithRawResponse,
    DCVDelegationResourceWithStreamingResponse,
    AsyncDCVDelegationResourceWithStreamingResponse,
)

__all__ = [
    "DCVDelegationResource",
    "AsyncDCVDelegationResource",
    "DCVDelegationResourceWithRawResponse",
    "AsyncDCVDelegationResourceWithRawResponse",
    "DCVDelegationResourceWithStreamingResponse",
    "AsyncDCVDelegationResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .client import Client as Client
from .token_get_response import TokenGetResponse as TokenGetResponse
from .connection_delete_params import ConnectionDeleteParams as ConnectionDeleteParams
from .management_create_params import ManagementCreateParams as ManagementCreateParams
from .configuration_get_response import ConfigurationGetResponse as ConfigurationGetResponse
from .management_create_response import ManagementCreateResponse as ManagementCreateResponse
from .configuration_update_params import ConfigurationUpdateParams as ConfigurationUpdateParams
from .configuration_update_response import ConfigurationUpdateResponse as ConfigurationUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .sfu_get_response import SFUGetResponse as SFUGetResponse
from .sfu_create_params import SFUCreateParams as SFUCreateParams
from .sfu_list_response import SFUListResponse as SFUListResponse
from .sfu_update_params import SFUUpdateParams as SFUUpdateParams
from .turn_get_response import TURNGetResponse as TURNGetResponse
from .turn_create_params import TURNCreateParams as TURNCreateParams
from .turn_list_response import TURNListResponse as TURNListResponse
from .turn_update_params import TURNUpdateParams as TURNUpdateParams
from .sfu_create_response import SFUCreateResponse as SFUCreateResponse
from .sfu_delete_response import SFUDeleteResponse as SFUDeleteResponse
from .sfu_update_response import SFUUpdateResponse as SFUUpdateResponse
from .turn_create_response import TURNCreateResponse as TURNCreateResponse
from .turn_delete_response import TURNDeleteResponse as TURNDeleteResponse
from .turn_update_response import TURNUpdateResponse as TURNUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ...._models import BaseModel

__all__ = ["PrefixBulkCreateResponse"]


class PrefixBulkCreateResponse(BaseModel):
    id: str
    """The unique ID of the prefix."""

    comment: str
    """A comment describing the prefix."""

    created_on: datetime
    """The creation timestamp of the prefix."""

    excluded: bool
    """Whether to exclude the prefix from protection."""

    modified_on: datetime
    """The last modification timestamp of the prefix."""

    prefix: str
    """The prefix in CIDR format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .layer3_summary_v2_params import Layer3SummaryV2Params as Layer3SummaryV2Params
from .layer3_timeseries_params import Layer3TimeseriesParams as Layer3TimeseriesParams
from .layer7_summary_v2_params import Layer7SummaryV2Params as Layer7SummaryV2Params
from .layer7_timeseries_params import Layer7TimeseriesParams as Layer7TimeseriesParams
from .layer3_summary_v2_response import Layer3SummaryV2Response as Layer3SummaryV2Response
from .layer3_timeseries_response import Layer3TimeseriesResponse as Layer3TimeseriesResponse
from .layer7_summary_v2_response import Layer7SummaryV2Response as Layer7SummaryV2Response
from .layer7_timeseries_response import Layer7TimeseriesResponse as Layer7TimeseriesResponse
from .layer3_timeseries_groups_v2_params import Layer3TimeseriesGroupsV2Params as Layer3TimeseriesGroupsV2Params
from .layer7_timeseries_groups_v2_params import Layer7TimeseriesGroupsV2Params as Layer7TimeseriesGroupsV2Params
from .layer3_timeseries_groups_v2_response import Layer3TimeseriesGroupsV2Response as Layer3TimeseriesGroupsV2Response
from .layer7_timeseries_groups_v2_response import Layer7TimeseriesGroupsV2Response as Layer7TimeseriesGroupsV2Response



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .widgets import (
    WidgetsResource,
    AsyncWidgetsResource,
    WidgetsResourceWithRawResponse,
    AsyncWidgetsResourceWithRawResponse,
    WidgetsResourceWithStreamingResponse,
    AsyncWidgetsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["TurnstileResource", "AsyncTurnstileResource"]


class TurnstileResource(SyncAPIResource):
    @cached_property
    def widgets(self) -> WidgetsResource:
        return WidgetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TurnstileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TurnstileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TurnstileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TurnstileResourceWithStreamingResponse(self)


class AsyncTurnstileResource(AsyncAPIResource):
    @cached_property
    def widgets(self) -> AsyncWidgetsResource:
        return AsyncWidgetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTurnstileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTurnstileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTurnstileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTurnstileResourceWithStreamingResponse(self)


class TurnstileResourceWithRawResponse:
    def __init__(self, turnstile: TurnstileResource) -> None:
        self._turnstile = turnstile

    @cached_property
    def widgets(self) -> WidgetsResourceWithRawResponse:
        return WidgetsResourceWithRawResponse(self._turnstile.widgets)


class AsyncTurnstileResourceWithRawResponse:
    def __init__(self, turnstile: AsyncTurnstileResource) -> None:
        self._turnstile = turnstile

    @cached_property
    def widgets(self) -> AsyncWidgetsResourceWithRawResponse:
        return AsyncWidgetsResourceWithRawResponse(self._turnstile.widgets)


class TurnstileResourceWithStreamingResponse:
    def __init__(self, turnstile: TurnstileResource) -> None:
        self._turnstile = turnstile

    @cached_property
    def widgets(self) -> WidgetsResourceWithStreamingResponse:
        return WidgetsResourceWithStreamingResponse(self._turnstile.widgets)


class AsyncTurnstileResourceWithStreamingResponse:
    def __init__(self, turnstile: AsyncTurnstileResource) -> None:
        self._turnstile = turnstile

    @cached_property
    def widgets(self) -> AsyncWidgetsResourceWithStreamingResponse:
        return AsyncWidgetsResourceWithStreamingResponse(self._turnstile.widgets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["InvestigateGetParams"]


class InvestigateGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    submission: bool
    """When true, search the submissions datastore only.

    When false or omitted, search the regular datastore only.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .r2 import (
    R2Resource,
    AsyncR2Resource,
    R2ResourceWithRawResponse,
    AsyncR2ResourceWithRawResponse,
    R2ResourceWithStreamingResponse,
    AsyncR2ResourceWithStreamingResponse,
)
from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .super_slurper import (
    SuperSlurperResource,
    AsyncSuperSlurperResource,
    SuperSlurperResourceWithRawResponse,
    AsyncSuperSlurperResourceWithRawResponse,
    SuperSlurperResourceWithStreamingResponse,
    AsyncSuperSlurperResourceWithStreamingResponse,
)
from .temporary_credentials import (
    TemporaryCredentialsResource,
    AsyncTemporaryCredentialsResource,
    TemporaryCredentialsResourceWithRawResponse,
    AsyncTemporaryCredentialsResourceWithRawResponse,
    TemporaryCredentialsResourceWithStreamingResponse,
    AsyncTemporaryCredentialsResourceWithStreamingResponse,
)

__all__ = [
    "BucketsResource",
    "AsyncBucketsResource",
    "BucketsResourceWithRawResponse",
    "AsyncBucketsResourceWithRawResponse",
    "BucketsResourceWithStreamingResponse",
    "AsyncBucketsResourceWithStreamingResponse",
    "TemporaryCredentialsResource",
    "AsyncTemporaryCredentialsResource",
    "TemporaryCredentialsResourceWithRawResponse",
    "AsyncTemporaryCredentialsResourceWithRawResponse",
    "TemporaryCredentialsResourceWithStreamingResponse",
    "AsyncTemporaryCredentialsResourceWithStreamingResponse",
    "SuperSlurperResource",
    "AsyncSuperSlurperResource",
    "SuperSlurperResourceWithRawResponse",
    "AsyncSuperSlurperResourceWithRawResponse",
    "SuperSlurperResourceWithStreamingResponse",
    "AsyncSuperSlurperResourceWithStreamingResponse",
    "R2Resource",
    "AsyncR2Resource",
    "R2ResourceWithRawResponse",
    "AsyncR2ResourceWithRawResponse",
    "R2ResourceWithStreamingResponse",
    "AsyncR2ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from .health_check_param import HealthCheckParam

__all__ = ["CfInterconnectUpdateParams", "GRE"]


class CfInterconnectUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    automatic_return_routing: bool
    """
    True if automatic stateful return routing should be enabled for a tunnel, false
    otherwise. Requires the `coupler_integration` account flag to be enabled;
    requests setting this to `true` without that flag will be rejected.
    """

    description: str
    """An optional description of the interconnect."""

    gre: GRE
    """The configuration specific to GRE interconnects."""

    health_check: HealthCheckParam

    interface_address: str
    """
    A 31-bit prefix (/31 in CIDR notation) supporting two hosts, one for each side
    of the tunnel. Select the subnet from the following private IP space:
    10.0.0.0–10.255.255.255, 172.16.0.0–172.31.255.255, 192.168.0.0–192.168.255.255.
    """

    interface_address6: str
    """
    A 127 bit IPV6 prefix from within the virtual_subnet6 prefix space with the
    address being the first IP of the subnet and not same as the address of
    virtual_subnet6. Eg if virtual_subnet6 is 2606:54c1:7:0:a9fe:12d2::/127 ,
    interface_address6 could be 2606:54c1:7:0:a9fe:12d2:1:200/127
    """

    mtu: int
    """The Maximum Transmission Unit (MTU) in bytes for the interconnect.

    The minimum value is 576.
    """

    name: str
    """The name of the interconnect. The name cannot share a name with other tunnels."""

    x_magic_new_hc_target: Annotated[bool, PropertyInfo(alias="x-magic-new-hc-target")]


class GRE(TypedDict, total=False):
    """The configuration specific to GRE interconnects."""

    cloudflare_endpoint: str
    """
    The IP address assigned to the Cloudflare side of the GRE tunnel created as part
    of the Interconnect.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AccountUpdateParams", "ManagedBy", "Settings"]


class AccountUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    id: Required[str]
    """Identifier"""

    name: Required[str]
    """Account name"""

    type: Required[Literal["standard", "enterprise"]]

    managed_by: ManagedBy
    """Parent container details"""

    settings: Settings
    """Account settings"""


class ManagedBy(TypedDict, total=False):
    """Parent container details"""

    pass


class Settings(TypedDict, total=False):
    """Account settings"""

    abuse_contact_email: str
    """Sets an abuse contact email to notify for abuse reports."""

    enforce_twofactor: bool
    """
    Indicates whether membership in this account requires that Two-Factor
    Authentication is enabled
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ThreatEventListResponse", "ThreatEventListResponseItem"]


class ThreatEventListResponseItem(BaseModel):
    attacker: str

    attacker_country: str = FieldInfo(alias="attackerCountry")

    category: str

    dataset_id: str = FieldInfo(alias="datasetId")

    date: str

    event: str

    has_children: bool = FieldInfo(alias="hasChildren")

    indicator: str

    indicator_type: str = FieldInfo(alias="indicatorType")

    indicator_type_id: float = FieldInfo(alias="indicatorTypeId")

    kill_chain: float = FieldInfo(alias="killChain")

    mitre_attack: List[str] = FieldInfo(alias="mitreAttack")

    mitre_capec: List[str] = FieldInfo(alias="mitreCapec")

    num_referenced: float = FieldInfo(alias="numReferenced")

    num_references: float = FieldInfo(alias="numReferences")

    raw_id: str = FieldInfo(alias="rawId")

    referenced: List[str]

    referenced_ids: List[float] = FieldInfo(alias="referencedIds")

    references: List[str]

    references_ids: List[float] = FieldInfo(alias="referencesIds")

    tags: List[str]

    target_country: str = FieldInfo(alias="targetCountry")

    target_industry: str = FieldInfo(alias="targetIndustry")

    tlp: str

    uuid: str

    insight: Optional[str] = None

    releasability_id: Optional[str] = FieldInfo(alias="releasabilityId", default=None)


ThreatEventListResponse: TypeAlias = List[ThreatEventListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .url_normalization_get_response import URLNormalizationGetResponse as URLNormalizationGetResponse
from .url_normalization_update_params import URLNormalizationUpdateParams as URLNormalizationUpdateParams
from .url_normalization_update_response import URLNormalizationUpdateResponse as URLNormalizationUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .subdomain_get_response import SubdomainGetResponse as SubdomainGetResponse
from .subdomain_create_params import SubdomainCreateParams as SubdomainCreateParams
from .subdomain_list_response import SubdomainListResponse as SubdomainListResponse
from .email_sending_send_params import EmailSendingSendParams as EmailSendingSendParams
from .subdomain_create_response import SubdomainCreateResponse as SubdomainCreateResponse
from .subdomain_delete_response import SubdomainDeleteResponse as SubdomainDeleteResponse
from .email_sending_send_response import EmailSendingSendResponse as EmailSendingSendResponse
from .email_sending_send_raw_params import EmailSendingSendRawParams as EmailSendingSendRawParams
from .email_sending_send_raw_response import EmailSendingSendRawResponse as EmailSendingSendRawResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AccessDevicePostureRuleParam", "DevicePosture"]


class DevicePosture(TypedDict, total=False):
    integration_uid: Required[str]
    """The ID of a device posture integration."""


class AccessDevicePostureRuleParam(TypedDict, total=False):
    """Enforces a device posture rule has run successfully"""

    device_posture: Required[DevicePosture]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["OperationUpdateParams"]


class OperationUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    mitigation_action: Required[Optional[Literal["log", "block", "none"]]]
    """When set, this applies a mitigation action to this operation

    - `"log"` - log request when request does not conform to schema for this
      operation
    - `"block"` - deny access to the site when request does not conform to schema
      for this operation
    - `"none"` - will skip mitigation for this operation
    - `null` - clears any mitigation action
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["NeoEventParam"]


class NeoEventParam(TypedDict, total=False):
    action_type: Required[Annotated[str, PropertyInfo(alias="actionType")]]
    """Tool event type."""

    blocking_triggers: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="blockingTriggers")]]
    """List of blocking triggers IDs."""

    data: Required[object]
    """Event payload."""

    firing_triggers: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="firingTriggers")]]
    """List of firing triggers IDs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ScriptDeleteParams"]


class ScriptDeleteParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    dispatch_namespace: Required[str]
    """Name of the Workers for Platforms dispatch namespace."""

    force: bool
    """
    If set to true, delete will not be stopped by associated service binding,
    durable object, or other binding. Any of these associated bindings/durable
    objects will be deleted along with the script.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CertificateAsssociation"]


class CertificateAsssociation(BaseModel):
    service: Optional[str] = None
    """The service using the certificate."""

    status: Optional[str] = None
    """Certificate deployment status for the given service."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.organizations import doh_update_params
from ....types.zero_trust.organizations.doh_get_response import DOHGetResponse
from ....types.zero_trust.organizations.doh_update_response import DOHUpdateResponse

__all__ = ["DOHResource", "AsyncDOHResource"]


class DOHResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DOHResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DOHResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DOHResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DOHResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        doh_jwt_duration: str | Omit = omit,
        service_token_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DOHUpdateResponse]:
        """
        Updates the DoH settings for your Zero Trust organization.

        Args:
          account_id: Identifier.

          doh_jwt_duration: The duration the DoH JWT is valid for. Must be in the format `300ms` or `2h45m`.
              Valid time units are: ns, us (or µs), ms, s, m, h. Note that the maximum
              duration for this setting is the same as the key rotation period on the account.
              Default expiration is 24h

          service_token_id: The uuid of the service token you want to use for DoH authentication

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/access/organizations/doh", account_id=account_id),
            body=maybe_transform(
                {
                    "doh_jwt_duration": doh_jwt_duration,
                    "service_token_id": service_token_id,
                },
                doh_update_params.DOHUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DOHUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DOHUpdateResponse]], ResultWrapper[DOHUpdateResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DOHGetResponse]:
        """
        Returns the DoH settings for your Zero Trust organization.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/access/organizations/doh", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DOHGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DOHGetResponse]], ResultWrapper[DOHGetResponse]),
        )


class AsyncDOHResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDOHResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDOHResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDOHResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDOHResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        doh_jwt_duration: str | Omit = omit,
        service_token_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DOHUpdateResponse]:
        """
        Updates the DoH settings for your Zero Trust organization.

        Args:
          account_id: Identifier.

          doh_jwt_duration: The duration the DoH JWT is valid for. Must be in the format `300ms` or `2h45m`.
              Valid time units are: ns, us (or µs), ms, s, m, h. Note that the maximum
              duration for this setting is the same as the key rotation period on the account.
              Default expiration is 24h

          service_token_id: The uuid of the service token you want to use for DoH authentication

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/access/organizations/doh", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "doh_jwt_duration": doh_jwt_duration,
                    "service_token_id": service_token_id,
                },
                doh_update_params.DOHUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DOHUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DOHUpdateResponse]], ResultWrapper[DOHUpdateResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DOHGetResponse]:
        """
        Returns the DoH settings for your Zero Trust organization.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/access/organizations/doh", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DOHGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DOHGetResponse]], ResultWrapper[DOHGetResponse]),
        )


class DOHResourceWithRawResponse:
    def __init__(self, doh: DOHResource) -> None:
        self._doh = doh

        self.update = to_raw_response_wrapper(
            doh.update,
        )
        self.get = to_raw_response_wrapper(
            doh.get,
        )


class AsyncDOHResourceWithRawResponse:
    def __init__(self, doh: AsyncDOHResource) -> None:
        self._doh = doh

        self.update = async_to_raw_response_wrapper(
            doh.update,
        )
        self.get = async_to_raw_response_wrapper(
            doh.get,
        )


class DOHResourceWithStreamingResponse:
    def __init__(self, doh: DOHResource) -> None:
        self._doh = doh

        self.update = to_streamed_response_wrapper(
            doh.update,
        )
        self.get = to_streamed_response_wrapper(
            doh.get,
        )


class AsyncDOHResourceWithStreamingResponse:
    def __init__(self, doh: AsyncDOHResource) -> None:
        self._doh = doh

        self.update = async_to_streamed_response_wrapper(
            doh.update,
        )
        self.get = async_to_streamed_response_wrapper(
            doh.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

from ..._models import BaseModel

__all__ = ["TraceCreateResponse"]


class TraceCreateResponse(BaseModel):
    """Trace result with an origin status code"""

    status_code: Optional[int] = None
    """HTTP Status code of zone response"""

    trace: Optional["Trace"] = None


from .trace import Trace



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, strip_not_given, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from .address_spaces import (
    AddressSpacesResource,
    AsyncAddressSpacesResource,
    AddressSpacesResourceWithRawResponse,
    AsyncAddressSpacesResourceWithRawResponse,
    AddressSpacesResourceWithStreamingResponse,
    AsyncAddressSpacesResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_cloud_networking import (
    on_ramp_get_params,
    on_ramp_edit_params,
    on_ramp_list_params,
    on_ramp_create_params,
    on_ramp_delete_params,
    on_ramp_update_params,
)
from ....types.magic_cloud_networking.on_ramp_get_response import OnRampGetResponse
from ....types.magic_cloud_networking.on_ramp_edit_response import OnRampEditResponse
from ....types.magic_cloud_networking.on_ramp_list_response import OnRampListResponse
from ....types.magic_cloud_networking.on_ramp_plan_response import OnRampPlanResponse
from ....types.magic_cloud_networking.on_ramp_apply_response import OnRampApplyResponse
from ....types.magic_cloud_networking.on_ramp_create_response import OnRampCreateResponse
from ....types.magic_cloud_networking.on_ramp_delete_response import OnRampDeleteResponse
from ....types.magic_cloud_networking.on_ramp_update_response import OnRampUpdateResponse

__all__ = ["OnRampsResource", "AsyncOnRampsResource"]


class OnRampsResource(SyncAPIResource):
    @cached_property
    def address_spaces(self) -> AddressSpacesResource:
        return AddressSpacesResource(self._client)

    @cached_property
    def with_raw_response(self) -> OnRampsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OnRampsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OnRampsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OnRampsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        cloud_type: Literal["AWS", "AZURE", "GOOGLE"],
        dynamic_routing: bool,
        install_routes_in_cloud: bool,
        install_routes_in_magic_wan: bool,
        name: str,
        type: Literal["OnrampTypeSingle", "OnrampTypeHub"],
        adopted_hub_id: str | Omit = omit,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        cloud_asn: int | Omit = omit,
        description: str | Omit = omit,
        hub_provider_id: str | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        region: str | Omit = omit,
        vpc: str | Omit = omit,
        forwarded: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampCreateResponse:
        """
        Create a new On-ramp (Closed Beta).

        Args:
          dynamic_routing: Enables BGP routing. When enabling this feature, set both
              install_routes_in_cloud and install_routes_in_magic_wan to false.

          cloud_asn: Sets the cloud-side ASN. If unset or zero, the cloud's default ASN takes effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {**strip_not_given({"forwarded": forwarded}), **(extra_headers or {})}
        return self._post(
            path_template("/accounts/{account_id}/magic/cloud/onramps", account_id=account_id),
            body=maybe_transform(
                {
                    "cloud_type": cloud_type,
                    "dynamic_routing": dynamic_routing,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "name": name,
                    "type": type,
                    "adopted_hub_id": adopted_hub_id,
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "cloud_asn": cloud_asn,
                    "description": description,
                    "hub_provider_id": hub_provider_id,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "region": region,
                    "vpc": vpc,
                },
                on_ramp_create_params.OnRampCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampCreateResponse], ResultWrapper[OnRampCreateResponse]),
        )

    def update(
        self,
        onramp_id: str,
        *,
        account_id: str,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        description: str | Omit = omit,
        install_routes_in_cloud: bool | Omit = omit,
        install_routes_in_magic_wan: bool | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        name: str | Omit = omit,
        vpc: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampUpdateResponse:
        """
        Update an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            body=maybe_transform(
                {
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "description": description,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "name": name,
                    "vpc": vpc,
                },
                on_ramp_update_params.OnRampUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampUpdateResponse], ResultWrapper[OnRampUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        desc: bool | Omit = omit,
        order_by: str | Omit = omit,
        status: bool | Omit = omit,
        vpcs: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[OnRampListResponse]:
        """
        List On-ramps (Closed Beta).

        Args:
          order_by: One of ["updated_at", "id", "cloud_type", "name"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/onramps", account_id=account_id),
            page=SyncSinglePage[OnRampListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "desc": desc,
                        "order_by": order_by,
                        "status": status,
                        "vpcs": vpcs,
                    },
                    on_ramp_list_params.OnRampListParams,
                ),
            ),
            model=OnRampListResponse,
        )

    def delete(
        self,
        onramp_id: str,
        *,
        account_id: str,
        destroy: bool | Omit = omit,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampDeleteResponse:
        """
        Delete an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "destroy": destroy,
                        "force": force,
                    },
                    on_ramp_delete_params.OnRampDeleteParams,
                ),
                post_parser=ResultWrapper[OnRampDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampDeleteResponse], ResultWrapper[OnRampDeleteResponse]),
        )

    def apply(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampApplyResponse:
        """
        Apply an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/apply",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OnRampApplyResponse,
        )

    def edit(
        self,
        onramp_id: str,
        *,
        account_id: str,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        description: str | Omit = omit,
        install_routes_in_cloud: bool | Omit = omit,
        install_routes_in_magic_wan: bool | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        name: str | Omit = omit,
        vpc: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampEditResponse:
        """
        Update an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            body=maybe_transform(
                {
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "description": description,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "name": name,
                    "vpc": vpc,
                },
                on_ramp_edit_params.OnRampEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampEditResponse], ResultWrapper[OnRampEditResponse]),
        )

    def export(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Export an On-ramp to terraform ready file(s) (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        extra_headers = {"Accept": "application/zip", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/export",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )

    def get(
        self,
        onramp_id: str,
        *,
        account_id: str,
        planned_resources: bool | Omit = omit,
        post_apply_resources: bool | Omit = omit,
        status: bool | Omit = omit,
        vpcs: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampGetResponse:
        """
        Read an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "planned_resources": planned_resources,
                        "post_apply_resources": post_apply_resources,
                        "status": status,
                        "vpcs": vpcs,
                    },
                    on_ramp_get_params.OnRampGetParams,
                ),
                post_parser=ResultWrapper[OnRampGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampGetResponse], ResultWrapper[OnRampGetResponse]),
        )

    def plan(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampPlanResponse:
        """
        Plan an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/plan",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OnRampPlanResponse,
        )


class AsyncOnRampsResource(AsyncAPIResource):
    @cached_property
    def address_spaces(self) -> AsyncAddressSpacesResource:
        return AsyncAddressSpacesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOnRampsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOnRampsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOnRampsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOnRampsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        cloud_type: Literal["AWS", "AZURE", "GOOGLE"],
        dynamic_routing: bool,
        install_routes_in_cloud: bool,
        install_routes_in_magic_wan: bool,
        name: str,
        type: Literal["OnrampTypeSingle", "OnrampTypeHub"],
        adopted_hub_id: str | Omit = omit,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        cloud_asn: int | Omit = omit,
        description: str | Omit = omit,
        hub_provider_id: str | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        region: str | Omit = omit,
        vpc: str | Omit = omit,
        forwarded: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampCreateResponse:
        """
        Create a new On-ramp (Closed Beta).

        Args:
          dynamic_routing: Enables BGP routing. When enabling this feature, set both
              install_routes_in_cloud and install_routes_in_magic_wan to false.

          cloud_asn: Sets the cloud-side ASN. If unset or zero, the cloud's default ASN takes effect.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {**strip_not_given({"forwarded": forwarded}), **(extra_headers or {})}
        return await self._post(
            path_template("/accounts/{account_id}/magic/cloud/onramps", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "cloud_type": cloud_type,
                    "dynamic_routing": dynamic_routing,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "name": name,
                    "type": type,
                    "adopted_hub_id": adopted_hub_id,
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "cloud_asn": cloud_asn,
                    "description": description,
                    "hub_provider_id": hub_provider_id,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "region": region,
                    "vpc": vpc,
                },
                on_ramp_create_params.OnRampCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampCreateResponse], ResultWrapper[OnRampCreateResponse]),
        )

    async def update(
        self,
        onramp_id: str,
        *,
        account_id: str,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        description: str | Omit = omit,
        install_routes_in_cloud: bool | Omit = omit,
        install_routes_in_magic_wan: bool | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        name: str | Omit = omit,
        vpc: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampUpdateResponse:
        """
        Update an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            body=await async_maybe_transform(
                {
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "description": description,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "name": name,
                    "vpc": vpc,
                },
                on_ramp_update_params.OnRampUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampUpdateResponse], ResultWrapper[OnRampUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        desc: bool | Omit = omit,
        order_by: str | Omit = omit,
        status: bool | Omit = omit,
        vpcs: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OnRampListResponse, AsyncSinglePage[OnRampListResponse]]:
        """
        List On-ramps (Closed Beta).

        Args:
          order_by: One of ["updated_at", "id", "cloud_type", "name"].

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/onramps", account_id=account_id),
            page=AsyncSinglePage[OnRampListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "desc": desc,
                        "order_by": order_by,
                        "status": status,
                        "vpcs": vpcs,
                    },
                    on_ramp_list_params.OnRampListParams,
                ),
            ),
            model=OnRampListResponse,
        )

    async def delete(
        self,
        onramp_id: str,
        *,
        account_id: str,
        destroy: bool | Omit = omit,
        force: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampDeleteResponse:
        """
        Delete an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "destroy": destroy,
                        "force": force,
                    },
                    on_ramp_delete_params.OnRampDeleteParams,
                ),
                post_parser=ResultWrapper[OnRampDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampDeleteResponse], ResultWrapper[OnRampDeleteResponse]),
        )

    async def apply(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampApplyResponse:
        """
        Apply an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/apply",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OnRampApplyResponse,
        )

    async def edit(
        self,
        onramp_id: str,
        *,
        account_id: str,
        attached_hubs: SequenceNotStr[str] | Omit = omit,
        attached_vpcs: SequenceNotStr[str] | Omit = omit,
        description: str | Omit = omit,
        install_routes_in_cloud: bool | Omit = omit,
        install_routes_in_magic_wan: bool | Omit = omit,
        manage_hub_to_hub_attachments: bool | Omit = omit,
        manage_vpc_to_hub_attachments: bool | Omit = omit,
        name: str | Omit = omit,
        vpc: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampEditResponse:
        """
        Update an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            body=await async_maybe_transform(
                {
                    "attached_hubs": attached_hubs,
                    "attached_vpcs": attached_vpcs,
                    "description": description,
                    "install_routes_in_cloud": install_routes_in_cloud,
                    "install_routes_in_magic_wan": install_routes_in_magic_wan,
                    "manage_hub_to_hub_attachments": manage_hub_to_hub_attachments,
                    "manage_vpc_to_hub_attachments": manage_vpc_to_hub_attachments,
                    "name": name,
                    "vpc": vpc,
                },
                on_ramp_edit_params.OnRampEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[OnRampEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampEditResponse], ResultWrapper[OnRampEditResponse]),
        )

    async def export(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Export an On-ramp to terraform ready file(s) (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        extra_headers = {"Accept": "application/zip", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/export",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )

    async def get(
        self,
        onramp_id: str,
        *,
        account_id: str,
        planned_resources: bool | Omit = omit,
        post_apply_resources: bool | Omit = omit,
        status: bool | Omit = omit,
        vpcs: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampGetResponse:
        """
        Read an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}", account_id=account_id, onramp_id=onramp_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "planned_resources": planned_resources,
                        "post_apply_resources": post_apply_resources,
                        "status": status,
                        "vpcs": vpcs,
                    },
                    on_ramp_get_params.OnRampGetParams,
                ),
                post_parser=ResultWrapper[OnRampGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[OnRampGetResponse], ResultWrapper[OnRampGetResponse]),
        )

    async def plan(
        self,
        onramp_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OnRampPlanResponse:
        """
        Plan an On-ramp (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not onramp_id:
            raise ValueError(f"Expected a non-empty value for `onramp_id` but received {onramp_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/onramps/{onramp_id}/plan",
                account_id=account_id,
                onramp_id=onramp_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OnRampPlanResponse,
        )


class OnRampsResourceWithRawResponse:
    def __init__(self, on_ramps: OnRampsResource) -> None:
        self._on_ramps = on_ramps

        self.create = to_raw_response_wrapper(
            on_ramps.create,
        )
        self.update = to_raw_response_wrapper(
            on_ramps.update,
        )
        self.list = to_raw_response_wrapper(
            on_ramps.list,
        )
        self.delete = to_raw_response_wrapper(
            on_ramps.delete,
        )
        self.apply = to_raw_response_wrapper(
            on_ramps.apply,
        )
        self.edit = to_raw_response_wrapper(
            on_ramps.edit,
        )
        self.export = to_custom_raw_response_wrapper(
            on_ramps.export,
            BinaryAPIResponse,
        )
        self.get = to_raw_response_wrapper(
            on_ramps.get,
        )
        self.plan = to_raw_response_wrapper(
            on_ramps.plan,
        )

    @cached_property
    def address_spaces(self) -> AddressSpacesResourceWithRawResponse:
        return AddressSpacesResourceWithRawResponse(self._on_ramps.address_spaces)


class AsyncOnRampsResourceWithRawResponse:
    def __init__(self, on_ramps: AsyncOnRampsResource) -> None:
        self._on_ramps = on_ramps

        self.create = async_to_raw_response_wrapper(
            on_ramps.create,
        )
        self.update = async_to_raw_response_wrapper(
            on_ramps.update,
        )
        self.list = async_to_raw_response_wrapper(
            on_ramps.list,
        )
        self.delete = async_to_raw_response_wrapper(
            on_ramps.delete,
        )
        self.apply = async_to_raw_response_wrapper(
            on_ramps.apply,
        )
        self.edit = async_to_raw_response_wrapper(
            on_ramps.edit,
        )
        self.export = async_to_custom_raw_response_wrapper(
            on_ramps.export,
            AsyncBinaryAPIResponse,
        )
        self.get = async_to_raw_response_wrapper(
            on_ramps.get,
        )
        self.plan = async_to_raw_response_wrapper(
            on_ramps.plan,
        )

    @cached_property
    def address_spaces(self) -> AsyncAddressSpacesResourceWithRawResponse:
        return AsyncAddressSpacesResourceWithRawResponse(self._on_ramps.address_spaces)


class OnRampsResourceWithStreamingResponse:
    def __init__(self, on_ramps: OnRampsResource) -> None:
        self._on_ramps = on_ramps

        self.create = to_streamed_response_wrapper(
            on_ramps.create,
        )
        self.update = to_streamed_response_wrapper(
            on_ramps.update,
        )
        self.list = to_streamed_response_wrapper(
            on_ramps.list,
        )
        self.delete = to_streamed_response_wrapper(
            on_ramps.delete,
        )
        self.apply = to_streamed_response_wrapper(
            on_ramps.apply,
        )
        self.edit = to_streamed_response_wrapper(
            on_ramps.edit,
        )
        self.export = to_custom_streamed_response_wrapper(
            on_ramps.export,
            StreamedBinaryAPIResponse,
        )
        self.get = to_streamed_response_wrapper(
            on_ramps.get,
        )
        self.plan = to_streamed_response_wrapper(
            on_ramps.plan,
        )

    @cached_property
    def address_spaces(self) -> AddressSpacesResourceWithStreamingResponse:
        return AddressSpacesResourceWithStreamingResponse(self._on_ramps.address_spaces)


class AsyncOnRampsResourceWithStreamingResponse:
    def __init__(self, on_ramps: AsyncOnRampsResource) -> None:
        self._on_ramps = on_ramps

        self.create = async_to_streamed_response_wrapper(
            on_ramps.create,
        )
        self.update = async_to_streamed_response_wrapper(
            on_ramps.update,
        )
        self.list = async_to_streamed_response_wrapper(
            on_ramps.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            on_ramps.delete,
        )
        self.apply = async_to_streamed_response_wrapper(
            on_ramps.apply,
        )
        self.edit = async_to_streamed_response_wrapper(
            on_ramps.edit,
        )
        self.export = async_to_custom_streamed_response_wrapper(
            on_ramps.export,
            AsyncStreamedBinaryAPIResponse,
        )
        self.get = async_to_streamed_response_wrapper(
            on_ramps.get,
        )
        self.plan = async_to_streamed_response_wrapper(
            on_ramps.plan,
        )

    @cached_property
    def address_spaces(self) -> AsyncAddressSpacesResourceWithStreamingResponse:
        return AsyncAddressSpacesResourceWithStreamingResponse(self._on_ramps.address_spaces)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["LogoMatchGetParams"]


class LogoMatchGetParams(TypedDict, total=False):
    account_id: Required[str]

    query_id: Required[str]

    download: str

    limit: str

    offset: str

    order: Literal["asc", "desc"]
    """Sort order. Options: 'asc' (ascending) or 'desc' (descending)"""

    order_by: Annotated[Literal["matchedAt", "domain", "similarityScore", "registrar"], PropertyInfo(alias="orderBy")]
    """Column to sort by.

    Options: 'matchedAt', 'domain', 'similarityScore', or 'registrar'
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from ..._models import BaseModel

__all__ = ["DomainHistory", "Categorization", "CategorizationCategory"]


class CategorizationCategory(BaseModel):
    id: Optional[int] = None

    name: Optional[str] = None


class Categorization(BaseModel):
    categories: Optional[List[CategorizationCategory]] = None

    end: Optional[date] = None

    start: Optional[date] = None


class DomainHistory(BaseModel):
    categorizations: Optional[List[Categorization]] = None

    domain: Optional[str] = None
